--
-- Prosody IM
-- Portions Copyright (C) 2010 Waqas Hussain
-- Portions Copyright (C) 2010 Jeff Mitchell
-- Portions Copyright (C) 2013 Mikael Nordfeldth
-- Portions Copyright (C) 2013 Matthew Wild, finally came to fix it all
-- Portions Copyright (C) 2014 Schmooze Com, Inc
--
-- This project is MIT/X11 licensed. Please see the
-- COPYING file in the source package for more information.
--

local usermanager = require "core.usermanager";
local new_sasl = require "util.sasl".new;
local server = require "net.server";
local have_async, async = pcall(require, "util.async");

local log = module._log;
local host = module.host;

local script_type = module:get_option_string("freepbx_auth_protocol", "userman");
local command = module:get_option_string("freepbx_auth_command", "");
local read_timeout = module:get_option_number("freepbx_auth_timeout", 5);
local blocking = module:get_option_boolean("freepbx_auth_blocking", not(have_async and server.event and lpty.getfd));
local auth_processes = module:get_option_number("freepbx_auth_processes", 1);

assert(script_type == "userman", "Config error: freepbx_auth_protocol must be 'userman'");
assert(not host:find(":"), "Invalid hostname");

if not blocking then
	log("debug", "External auth in non-blocking mode, yay!")
	waiter, guard = async.waiter, async.guarder();
elseif auth_processes > 1 then
	log("warn", "freepbx_auth_processes is greater than 1, but we are in blocking mode - reducing to 1");
	auth_processes = 1;
end

function send_query(text)
	if blocking then
		local handle = io.popen(command .. " " .. text)
		local result = handle:read("*a")
		handle:close()
		return result;
	else
		local handle = io.popen(command .. " " .. text)
		local result = handle:read("*a")
		handle:close()
		return result;
	end
end

function do_query(kind, username, password)
	if not username then return nil, "not-acceptable"; end

	local query = (password and "%s:%s:%s:%s" or "%s:%s:%s"):format(kind, username, host, password);
	local len = #query
	if len > 1000 then return nil, "policy-violation"; end

	local response, err = send_query(query);
	if not response then
		log("warn", "Error while waiting for result from auth process: %s", err or "unknown error");
	elseif (script_type == "userman" and response:gsub("\r?\n$", "") == "0") then
			return nil, "not-authorized";
	elseif (script_type == "userman" and response:gsub("\r?\n$", "") == "1") then
			return true;
	else
		log("warn", "Unable to interpret data from auth process, %s", (response:match("^error:") and response) or ("["..#response.." bytes]"));
		return nil, "internal-server-error";
	end
end

local host = module.host;
local provider = {};

function provider.test_password(username, password)
	return do_query("auth", username, password);
end

function provider.set_password(username, password)
	return do_query("setpass", username, password);
end

function provider.user_exists(username)
	return do_query("isuser", username);
end

function provider.get_sasl_handler()
	local getpass_authentication_profile = {
		plain_test = function(sasl, username, password, realm)
			return usermanager.test_password(username, realm, password), true;
		end
	};
	return new_sasl(host, getpass_authentication_profile);
end

function provider.create_user(username, password) return nil, "Account creation/modification not available."; end

module:provides("auth", provider);